<div class="row" style="background-color: #EAEAEA; border-top-right-radius: 15px; border-top-left-radius: 15px;">
    <p class="text-center footer-text" style="padding-top: 15px;"> Sergey Aganezov:
        <a href="mailto:aganezov@gwu.edu">aganezov@gwu.edu</a></p>
    <p class="text-center"> Dr. Max A. Alekseyev: <a href="mailto:maxal@gwu.edu">maxal@gwu.edu</a></p>
    <p class="text-center">The project is supported by the National Science Foundation under Grant No. IIS-1462107.</p>
    <p class="text-muted text-center">2016 - {{ meta.date.strftime('%Y') }}</p>
</div>