{% include "comparative/assemblies_overview_template.html" %}
{% include "comparative/grouped_assemblies_overview_template.html" %}
{% include "comparative/per_assembly_template.html" %}
{% include "comparative/aggregated_assembly_points_template.html" %}